@@ -30,15 +30,16 @@ using namespace AppStream;
 
 class AppStream::PoolPrivate {
     public:
-        AsPool *m_pool;
+        AsPool *pool;
+        QString lastError;
 
         PoolPrivate()
         {
-            m_pool = as_pool_new();
+            pool = as_pool_new();
         }
 
         ~PoolPrivate() {
-            g_object_unref(m_pool);
+            g_object_unref(pool);
         }
 };
 
@@ -66,103 +67,114 @@ Pool::~Pool()
 
 bool AppStream::Pool::load()
 {
-    return load(nullptr);
+    g_autoptr(GError) error = nullptr;
+    auto ret = as_pool_load (d->pool, NULL, &error);
+    if (!ret && error)
+        d->lastError = QString::fromUtf8(error->message);
+    return ret;
 }
 
 bool Pool::load(QString* strerror)
 {
-    g_autoptr(GError) error = nullptr;
-    bool ret = as_pool_load (d->m_pool, NULL, &error);
-    if (!ret && error && strerror) {
-        *strerror = QString::fromUtf8(error->message);
+    auto ret = load();
+    if (!ret && strerror) {
+        *strerror = d->lastError;
     }
     return ret;
 }
 
 void Pool::clear()
 {
-    return as_pool_clear (d->m_pool);
+    g_autoptr(GError) error = nullptr;
+    auto ret = as_pool_clear2 (d->pool, &error);
+    if (!ret && error)
+        d->lastError = QString::fromUtf8(error->message);
+}
+
+QString Pool::lastError() const
+{
+    return d->lastError;
 }
 
 bool Pool::addComponent(const AppStream::Component& cpt)
 {
     // FIXME: We ignore errors for now.
-    return as_pool_add_component (d->m_pool, cpt.m_cpt, NULL);
+    return as_pool_add_component (d->pool, cpt.m_cpt, NULL);
 }
 
 QList<Component> Pool::components() const
 {
-    return cptArrayToQList(as_pool_get_components(d->m_pool));
+    return cptArrayToQList(as_pool_get_components(d->pool));
 }
 
 QList<Component> Pool::componentsById(const QString& cid) const
 {
-    return cptArrayToQList(as_pool_get_components_by_id(d->m_pool, qPrintable(cid)));
+    return cptArrayToQList(as_pool_get_components_by_id(d->pool, qPrintable(cid)));
 }
 
 QList<Component> Pool::componentsByProvided(Provided::Kind kind, const QString& item) const
 {
-    return cptArrayToQList(as_pool_get_components_by_provided_item(d->m_pool,
+    return cptArrayToQList(as_pool_get_components_by_provided_item(d->pool,
                                                                    static_cast<AsProvidedKind>(kind),
                                                                    qPrintable(item)));
 }
 
 QList<AppStream::Component> Pool::componentsByKind(Component::Kind kind) const
 {
-    return cptArrayToQList(as_pool_get_components_by_kind(d->m_pool, static_cast<AsComponentKind>(kind)));
+    return cptArrayToQList(as_pool_get_components_by_kind(d->pool, static_cast<AsComponentKind>(kind)));
 }
 
 QList<AppStream::Component> Pool::componentsByCategories(const QStringList categories) const
 {
     // FIXME: Todo
     QList<AppStream::Component> res;
-    //! return cptArrayToQList(as_pool_get_components_by_categories (d->m_pool, );
+    //! return cptArrayToQList(as_pool_get_components_by_categories (d->pool, );
     return res;
 }
 
 QList<Component> Pool::componentsByLaunchable(Launchable::Kind kind, const QString& value) const
 {
-    return cptArrayToQList(as_pool_get_components_by_launchable(d->m_pool,
+    return cptArrayToQList(as_pool_get_components_by_launchable(d->pool,
                                                                    static_cast<AsLaunchableKind>(kind),
                                                                    qPrintable(value)));
 }
 
 QList<AppStream::Component> Pool::search(const QString& term) const
 {
-    return cptArrayToQList(as_pool_search(d->m_pool, qPrintable(term)));
+    return cptArrayToQList(as_pool_search(d->pool, qPrintable(term)));
 }
 
 void Pool::clearMetadataLocations()
 {
-    as_pool_clear_metadata_locations(d->m_pool);
+    as_pool_clear_metadata_locations(d->pool);
 }
 
 void Pool::addMetadataLocation(const QString& directory)
 {
-    as_pool_add_metadata_location (d->m_pool, qPrintable(directory));
+    as_pool_add_metadata_location (d->pool, qPrintable(directory));
 }
 
 void Pool::setLocale(const QString& locale)
 {
-    as_pool_set_locale (d->m_pool, qPrintable(locale));
+    as_pool_set_locale (d->pool, qPrintable(locale));
 }
 
 uint Pool::flags() const
 {
-    return (uint) as_pool_get_flags(d->m_pool);
+    return (uint) as_pool_get_flags(d->pool);
 }
 
 void Pool::setFlags(uint flags)
 {
-    as_pool_set_flags (d->m_pool, (AsPoolFlags) flags);
+    as_pool_set_flags (d->pool, (AsPoolFlags) flags);
 }
 
 uint Pool::cacheFlags() const
 {
-    return (uint) as_pool_get_cache_flags(d->m_pool);
+    return (uint) as_pool_get_cache_flags(d->pool);
 }
 
 void Pool::setCacheFlags(uint flags)
 {
-    as_pool_set_cache_flags (d->m_pool, (AsCacheFlags) flags);
+    as_pool_set_cache_flags (d->pool, (AsCacheFlags) flags);
 }
