@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2018-2019 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2018-2020 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -25,6 +25,7 @@
  * Caches are used by #AsPool to quickly search for components while not keeping all
  * component data in memory.
  * Internally, a cache is backed by an LMDB database.
+ * This class is threadsafe.
  *
  * See also: #AsPool
  */
@@ -37,7 +38,7 @@
 #include <glib/gstdio.h>
 
 #include "as-utils-private.h"
-#include "as-context.h"
+#include "as-context-private.h"
 #include "as-component-private.h"
 #include "as-launchable.h"
 
@@ -90,6 +91,8 @@ typedef struct
 
 	GFunc cpt_refine_func;
 	gpointer cpt_refine_func_udata;
+
+	GMutex mutex;
 } AsCachePrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsCache, as_cache, G_TYPE_OBJECT)
@@ -136,6 +139,8 @@ as_cache_init (AsCache *cache)
 {
 	AsCachePrivate *priv = GET_PRIVATE (cache);
 
+	g_mutex_init (&priv->mutex);
+
 	priv->opened = FALSE;
 	priv->max_keysize = 511;
 	priv->cpt_refine_func = NULL;
@@ -177,16 +182,19 @@ as_cache_finalize (GObject *object)
 	AsCache *cache = AS_CACHE (object);
 	AsCachePrivate *priv = GET_PRIVATE (cache);
 
-	g_object_unref (priv->context);
 	as_cache_close (cache);
+	g_mutex_lock (&priv->mutex);
+	g_object_unref (priv->context);
 	g_free (priv->locale);
-
 	g_free (priv->fname);
 
 	g_hash_table_unref (priv->cpt_map);
 	g_hash_table_unref (priv->cid_set);
 	g_hash_table_unref (priv->ro_removed_set);
 
+	g_mutex_unlock (&priv->mutex);
+	g_mutex_clear (&priv->mutex);
+
 	G_OBJECT_CLASS (as_cache_parent_class)->finalize (object);
 }
 
@@ -340,7 +348,7 @@ lmdb_val_memdup (MDB_val val, gsize *len)
  *
  * Add key/value pair to the database
  */
-inline static gboolean
+static gboolean
 as_cache_txn_put_kv (AsCache *cache, MDB_txn *txn, MDB_dbi dbi, const gchar *key, MDB_val dval, GError **error)
 {
 	AsCachePrivate *priv = GET_PRIVATE (cache);
@@ -349,8 +357,18 @@ as_cache_txn_put_kv (AsCache *cache, MDB_txn *txn, MDB_dbi dbi, const gchar *key
 	gsize key_len;
 	g_autofree gchar *key_hash = NULL;
 
-	/* if key is too long, hash it */
+	/* if key is too short, return error */
+	g_assert (key != NULL);
 	key_len = sizeof(gchar) * strlen (key);
+	if (key_len == 0) {
+		g_set_error (error,
+			     AS_CACHE_ERROR,
+			     AS_CACHE_ERROR_BAD_DATA,
+			     "Can not add an empty (zero-length) key to the cache");
+		return FALSE;
+	}
+
+	/* if key is too long, hash it */
 	if (key_len > priv->max_keysize) {
 		key_hash = g_compute_checksum_for_string (AS_CACHE_CHECKSUM, key, key_len);
 		dkey = lmdb_str_to_dbval (key_hash, -1);
@@ -628,6 +646,7 @@ inline static gboolean
 as_cache_check_opened (AsCache *cache, gboolean allow_floating, GError **error)
 {
 	AsCachePrivate *priv = GET_PRIVATE (cache);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 
 	if (!allow_floating && priv->floating) {
 		g_set_error (error,
@@ -703,10 +722,13 @@ as_cache_open (AsCache *cache, const gchar *fname, const gchar *locale, GError *
 	mdb_mode_t db_mode;
 	gboolean nosync;
 	gboolean readonly;
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	/* close cache in case it was open */
 	as_cache_close (cache);
 
+	locker = g_mutex_locker_new (&priv->mutex);
+
 	rc = mdb_env_create (&priv->db_env);
 	if (rc != MDB_SUCCESS) {
 		g_set_error (error,
@@ -995,6 +1017,7 @@ gboolean
 as_cache_close (AsCache *cache)
 {
 	AsCachePrivate *priv = GET_PRIVATE (cache);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 
 	if (!priv->opened)
 		return FALSE;
@@ -1099,11 +1122,11 @@ as_cache_insert (AsCache *cache, AsComponent *cpt, GError **error)
 	GPtrArray *provides;
 	GPtrArray *extends;
 
-	static GMutex mutex;
-	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&mutex);
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	if (!as_cache_check_opened (cache, TRUE, error))
 		return FALSE;
+	locker = g_mutex_locker_new (&priv->mutex);
 
 	if (priv->floating) {
 		/* floating cache, don't really add this component yet but stage it in the internal map */
@@ -1158,20 +1181,27 @@ as_cache_insert (AsCache *cache, AsComponent *cpt, GError **error)
 		g_autofree guint8 *match_list = NULL;
 		gsize match_list_len;
 		const gchar *token_str;
+		size_t token_len;
 		AsTokenType *match_pval;
 
 		/* we ignore tokens which are too long to be database keys */
 		token_str = (const gchar*) tc_key;
-		if ((sizeof(gchar) * strlen (token_str)) > priv->max_keysize) {
+		token_len = sizeof(gchar) * strlen (token_str);
+		if (token_len == 0) {
+			g_warning ("Ignored empty search token for component '%s'", as_component_get_data_id (cpt));
+			continue;
+		}
+		if (token_len > priv->max_keysize) {
 			g_warning ("Ignored search token '%s': Too long to be stored in the cache.", token_str);
 			continue;
 		}
 
+		/* get existing fts match value - we deliberately ignore any errors while reading here */
 		fts_val = as_cache_txn_get_value (cache,
 						  txn,
 						  priv->db_fts,
 						  token_str,
-						  &tmp_error);
+						  NULL);
 
 		match_pval = (AsTokenType *) tc_value;
 		match_list = g_memdup (fts_val.mv_data, fts_val.mv_size); /* TODO: There is potential to save on allocations here */
@@ -1423,9 +1453,11 @@ as_cache_remove_by_data_id (AsCache *cache, const gchar *cdid, GError **error)
 	g_autofree guint8 *cpt_checksum = NULL;
 	GError *tmp_error = NULL;
 	gboolean ret;
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	if (!as_cache_check_opened (cache, TRUE, error))
 		return FALSE;
+	locker = g_mutex_locker_new (&priv->mutex);
 
 	if (priv->floating) {
 		/* floating cache, remove only from the internal map */
@@ -1649,9 +1681,11 @@ as_cache_get_components_all (AsCache *cache, GError **error)
 	MDB_val dval;
 	MDB_val dkey;
 	g_autoptr(GPtrArray) results = NULL;
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	if (!as_cache_check_opened (cache, FALSE, error))
 		return NULL;
+	locker = g_mutex_locker_new (&priv->mutex);
 
 	txn = as_cache_transaction_new (cache, MDB_RDONLY, error);
 	if (txn == NULL)
@@ -1698,7 +1732,7 @@ as_cache_get_components_all (AsCache *cache, GError **error)
 }
 
 /**
- * as_cache_get_component_by_cid:
+ * as_cache_get_components_by_id:
  * @cache: An instance of #AsCache.
  * @id: The component ID to search for.
  * @error: A #GError or %NULL.
@@ -1715,9 +1749,11 @@ as_cache_get_components_by_id (AsCache *cache, const gchar *id, GError **error)
 	GError *tmp_error = NULL;
 	MDB_val dval;
 	GPtrArray *result = NULL;
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	if (!as_cache_check_opened (cache, TRUE, error))
 		return NULL;
+	locker = g_mutex_locker_new (&priv->mutex);
 
 	if (priv->floating) {
 		/* floating cache, check only the internal map */
@@ -1778,9 +1814,11 @@ as_cache_get_component_by_data_id (AsCache *cache, const gchar *cdid, GError **e
 	GError *tmp_error = NULL;
 	g_autofree guint8 *cpt_hash = NULL;
 	AsComponent *cpt;
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	if (!as_cache_check_opened (cache, TRUE, error))
 		return NULL;
+	locker = g_mutex_locker_new (&priv->mutex);
 
 	if (priv->floating) {
 		/* floating cache, check only the internal map */
@@ -1830,9 +1868,11 @@ as_cache_get_components_by_kind (AsCache *cache, AsComponentKind kind, GError **
 	MDB_val dval;
 	GPtrArray *result = NULL;
 	const gchar *kind_str = as_component_kind_to_string (kind);
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	if (!as_cache_check_opened (cache, FALSE, error))
 		return NULL;
+	locker = g_mutex_locker_new (&priv->mutex);
 
 	txn = as_cache_transaction_new (cache, MDB_RDONLY, error);
 	if (txn == NULL)
@@ -1875,9 +1915,11 @@ as_cache_get_components_by_provided_item (AsCache *cache, AsProvidedKind kind, c
 	MDB_val dval;
 	g_autofree gchar *item_key = NULL;
 	GPtrArray *result = NULL;
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	if (!as_cache_check_opened (cache, FALSE, error))
 		return NULL;
+	locker = g_mutex_locker_new (&priv->mutex);
 
 	item_key = g_strconcat (as_provided_kind_to_string (kind), item, NULL);
 	txn = as_cache_transaction_new (cache, MDB_RDONLY, error);
@@ -1918,9 +1960,11 @@ as_cache_get_components_by_categories (AsCache *cache, gchar **categories, GErro
 	MDB_txn *txn;
 	GError *tmp_error = NULL;
 	g_autoptr(GPtrArray) result = NULL;
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	if (!as_cache_check_opened (cache, FALSE, error))
 		return NULL;
+	locker = g_mutex_locker_new (&priv->mutex);
 
 	txn = as_cache_transaction_new (cache, MDB_RDONLY, error);
 	if (txn == NULL)
@@ -1978,9 +2022,11 @@ as_cache_get_components_by_launchable (AsCache *cache, AsLaunchableKind kind, co
 	g_autofree gchar *entry_key = NULL;
 	MDB_val dval;
 	GPtrArray *result = NULL;
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	if (!as_cache_check_opened (cache, FALSE, error))
 		return NULL;
+	locker = g_mutex_locker_new (&priv->mutex);
 
 	entry_key = g_strconcat (as_launchable_kind_to_string (kind), id, NULL);
 	txn = as_cache_transaction_new (cache, MDB_RDONLY, error);
@@ -2049,6 +2095,10 @@ as_cache_update_results_with_fts_value (AsCache *cache, MDB_txn *txn, MDB_val dv
 					sort_score |= match_pval << 2;
 				else
 					sort_score |= match_pval;
+
+				if ((as_component_get_kind (cpt) == AS_COMPONENT_KIND_ADDON) && (match_pval > 0))
+					sort_score--;
+
 				as_component_set_sort_score (cpt, sort_score);
 
 				g_hash_table_insert (results_ht,
@@ -2061,6 +2111,12 @@ as_cache_update_results_with_fts_value (AsCache *cache, MDB_txn *txn, MDB_val dv
 				sort_score |= match_pval << 2;
 			else
 				sort_score |= match_pval;
+
+			if (as_component_get_sort_score (cpt) == 0) {
+				if ((as_component_get_kind (cpt) == AS_COMPONENT_KIND_ADDON))
+					sort_score--;
+			}
+
 			as_component_set_sort_score (cpt, sort_score);
 		}
 	}
@@ -2089,6 +2145,7 @@ as_cache_search (AsCache *cache, gchar **terms, gboolean sort, GError **error)
 	g_autoptr(GHashTable) results_ht = NULL;
 	GHashTableIter ht_iter;
 	gpointer ht_value;
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	if (!as_cache_check_opened (cache, FALSE, error))
 		return NULL;
@@ -2098,6 +2155,8 @@ as_cache_search (AsCache *cache, gchar **terms, gboolean sort, GError **error)
 		return as_cache_get_components_all (cache, error);
 	}
 
+	locker = g_mutex_locker_new (&priv->mutex);
+
 	txn = as_cache_transaction_new (cache, MDB_RDONLY, error);
 	if (txn == NULL)
 		return NULL;
@@ -2180,6 +2239,9 @@ as_cache_search (AsCache *cache, gchar **terms, gboolean sort, GError **error)
 		mdb_cursor_close (cur);
 	}
 
+	/* we don't need the mutex anymore, no class struct access here */
+	g_clear_pointer (&locker, g_mutex_locker_free);
+
 	/* compile our result */
 	g_hash_table_iter_init (&ht_iter, results_ht);
 	while (g_hash_table_iter_next (&ht_iter, NULL, &ht_value))
@@ -2211,9 +2273,11 @@ as_cache_has_component_id (AsCache *cache, const gchar *id, GError **error)
 	GError *tmp_error = NULL;
 	MDB_val dval;
 	gboolean found;
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	if (!as_cache_check_opened (cache, TRUE, error))
 		return FALSE;
+	locker = g_mutex_locker_new (&priv->mutex);
 
 	if (priv->floating) {
 		/* floating cache, check only the internal map */
@@ -2254,9 +2318,11 @@ as_cache_count_components (AsCache *cache, GError **error)
 	MDB_stat stats;
 	gint rc;
 	gssize count = -1;
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	if (!as_cache_check_opened (cache, FALSE, error))
 		return 0;
+	locker = g_mutex_locker_new (&priv->mutex);
 
 	txn = as_cache_transaction_new (cache, MDB_RDONLY, error);
 	if (txn == NULL)
@@ -2289,6 +2355,7 @@ as_cache_get_ctime (AsCache *cache)
 {
 	AsCachePrivate *priv = GET_PRIVATE (cache);
 	struct stat cache_sbuf;
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 
 	if (priv->fname == NULL)
 		return 0;
@@ -2309,6 +2376,7 @@ gboolean
 as_cache_is_open (AsCache *cache)
 {
 	AsCachePrivate *priv = GET_PRIVATE (cache);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 	return priv->opened;
 }
 
@@ -2323,6 +2391,7 @@ void
 as_cache_make_floating (AsCache *cache)
 {
 	AsCachePrivate *priv = GET_PRIVATE (cache);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 
 	if (priv->floating)
 		return;
@@ -2348,6 +2417,8 @@ as_cache_unfloat (AsCache *cache, GError **error)
 	gpointer ht_value;
 	guint invalid_cpts = 0;
 
+	g_mutex_lock (&priv->mutex);
+
 	priv->floating = FALSE;
 
 	g_hash_table_iter_init (&iter, priv->cpt_map);
@@ -2368,13 +2439,16 @@ as_cache_unfloat (AsCache *cache, GError **error)
 			continue;
 		}
 
+		g_mutex_unlock (&priv->mutex);
 		if (!as_cache_insert (cache, cpt, error))
 			return 0;
+		g_mutex_lock (&priv->mutex);
 	}
 
 	g_hash_table_remove_all (priv->cid_set);
 	g_hash_table_remove_all (priv->cpt_map);
 
+	g_mutex_unlock (&priv->mutex);
 	g_debug ("Cache returned from floating mode (all changes are now persistent)");
 
 	return invalid_cpts;
@@ -2390,6 +2464,7 @@ const gchar*
 as_cache_get_location (AsCache *cache)
 {
 	AsCachePrivate *priv = GET_PRIVATE (cache);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 	return priv->fname;
 }
 
@@ -2403,6 +2478,8 @@ void
 as_cache_set_location (AsCache *cache, const gchar *location)
 {
 	AsCachePrivate *priv = GET_PRIVATE (cache);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
+
 	g_free (priv->fname);
 	priv->fname = g_strdup (location);
 }
@@ -2417,6 +2494,7 @@ gboolean
 as_cache_get_nosync (AsCache *cache)
 {
 	AsCachePrivate *priv = GET_PRIVATE (cache);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 	return priv->nosync;
 }
 
@@ -2432,6 +2510,7 @@ void
 as_cache_set_nosync (AsCache *cache, gboolean nosync)
 {
 	AsCachePrivate *priv = GET_PRIVATE (cache);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 	priv->nosync = nosync;
 }
 
@@ -2445,6 +2524,7 @@ gboolean
 as_cache_get_readonly (AsCache *cache)
 {
 	AsCachePrivate *priv = GET_PRIVATE (cache);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 	return priv->readonly;
 }
 
@@ -2458,6 +2538,7 @@ void
 as_cache_set_readonly (AsCache *cache, gboolean ro)
 {
 	AsCachePrivate *priv = GET_PRIVATE (cache);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 	priv->readonly = ro;
 }
 
@@ -2471,6 +2552,8 @@ void
 as_cache_set_refine_func (AsCache *cache, GFunc func, gpointer user_data)
 {
 	AsCachePrivate *priv = GET_PRIVATE (cache);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
+
 	priv->cpt_refine_func = func;
 	priv->cpt_refine_func_udata = user_data;
 }
