@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2016 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2020 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -47,8 +47,10 @@ struct _AsDistroDetailsClass
 AsDistroDetails		*as_distro_details_new (void);
 
 const gchar		*as_distro_details_get_id (AsDistroDetails *distro);
+const gchar		*as_distro_details_get_cid (AsDistroDetails *distro);
 const gchar		*as_distro_details_get_name (AsDistroDetails *distro);
 const gchar		*as_distro_details_get_version (AsDistroDetails *distro);
+const gchar		*as_distro_details_get_homepage (AsDistroDetails *distro);
 
 gchar			*as_distro_details_get_str (AsDistroDetails *distro,
 							const gchar *key);
@@ -56,6 +58,8 @@ gboolean		as_distro_details_get_bool (AsDistroDetails *distro,
 							const gchar *key,
 							gboolean default_val);
 
+gchar 			*as_get_current_distro_component_id (void);
+
 G_END_DECLS
 
 #endif /* __AS_DISTRODETAILS_H */
