@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2016 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2020 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -40,9 +40,9 @@
 #include "as-utils-private.h"
 #include "as-component.h"
 #include "as-component-private.h"
+#include "as-context-private.h"
 #include "as-distro-details.h"
 #include "as-desktop-entry.h"
-#include "as-context.h"
 
 #include "as-xml.h"
 #include "as-yaml.h"
@@ -67,107 +67,6 @@ typedef struct
 G_DEFINE_TYPE_WITH_PRIVATE (AsMetadata, as_metadata, G_TYPE_OBJECT)
 #define GET_PRIVATE(o) (as_metadata_get_instance_private (o))
 
-/**
- * as_format_kind_to_string:
- * @kind: the #AsFormatKind.
- *
- * Converts the enumerated value to an text representation.
- *
- * Returns: string version of @kind
- *
- * Since: 0.10
- **/
-const gchar*
-as_format_kind_to_string (AsFormatKind kind)
-{
-	if (kind == AS_FORMAT_KIND_XML)
-		return "xml";
-	if (kind == AS_FORMAT_KIND_YAML)
-		return "yaml";
-	return "unknown";
-}
-
-/**
- * as_format_kind_from_string:
- * @kind_str: the string.
- *
- * Converts the text representation to an enumerated value.
- *
- * Returns: a #AsFormatKind or %AS_FORMAT_KIND_UNKNOWN for unknown
- *
- * Since: 0.10
- **/
-AsFormatKind
-as_format_kind_from_string (const gchar *kind_str)
-{
-	if (g_strcmp0 (kind_str, "xml") == 0)
-		return AS_FORMAT_KIND_XML;
-	if (g_strcmp0 (kind_str, "yaml") == 0)
-		return AS_FORMAT_KIND_YAML;
-	return AS_FORMAT_KIND_UNKNOWN;
-}
-
-/**
- * as_format_version_to_string:
- * @version: the #AsFormatKind.
- *
- * Converts the enumerated value to an text representation.
- *
- * Returns: string version of @version
- *
- * Since: 0.10
- **/
-const gchar*
-as_format_version_to_string (AsFormatVersion version)
-{
-	if (version == AS_FORMAT_VERSION_V0_6)
-		return "0.6";
-	if (version == AS_FORMAT_VERSION_V0_7)
-		return "0.7";
-	if (version == AS_FORMAT_VERSION_V0_8)
-		return "0.8";
-	if (version == AS_FORMAT_VERSION_V0_9)
-		return "0.9";
-	if (version == AS_FORMAT_VERSION_V0_10)
-		return "0.10";
-	if (version == AS_FORMAT_VERSION_V0_11)
-		return "0.11";
-	if (version == AS_FORMAT_VERSION_V0_12)
-		return "0.12";
-	return "?.??";
-}
-
-/**
- * as_format_version_from_string:
- * @version_str: the string.
- *
- * Converts the text representation to an enumerated value.
- *
- * Returns: a #AsFormatVersion. For unknown, the highest version
- * number is assumed.
- *
- * Since: 0.10
- **/
-AsFormatVersion
-as_format_version_from_string (const gchar *version_str)
-{
-	if (g_strcmp0 (version_str, "0.12") == 0)
-		return AS_FORMAT_VERSION_V0_12;
-	if (g_strcmp0 (version_str, "0.11") == 0)
-		return AS_FORMAT_VERSION_V0_11;
-	if (g_strcmp0 (version_str, "0.10") == 0)
-		return AS_FORMAT_VERSION_V0_10;
-	if (g_strcmp0 (version_str, "0.9") == 0)
-		return AS_FORMAT_VERSION_V0_9;
-	if (g_strcmp0 (version_str, "0.8") == 0)
-		return AS_FORMAT_VERSION_V0_8;
-	if (g_strcmp0 (version_str, "0.7") == 0)
-		return AS_FORMAT_VERSION_V0_7;
-	if (g_strcmp0 (version_str, "0.6") == 0)
-		return AS_FORMAT_VERSION_V0_6;
-	return AS_FORMAT_VERSION_V0_10;
-}
-
 /**
  * as_metadata_init:
  **/
