@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2016 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2020 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -30,6 +30,7 @@
 #include <gio/gio.h>
 
 #include "as-component.h"
+#include "as-context.h"
 
 G_BEGIN_DECLS
 
@@ -49,72 +50,16 @@ struct _AsMetadataClass
 };
 
 /**
- * AsFormatStyle:
- * @AS_FORMAT_STYLE_METAINFO:	Parse AppStream upstream metadata (metainfo files)
- * @AS_FORMAT_STYLE_COLLECTION:	Parse AppStream metadata collections (shipped by software distributors)
+ * AsParseFlags:
+ * @AS_PARSE_FLAG_NONE:				No flags.
+ * @AS_PARSE_FLAG_IGNORE_MEDIABASEURL:		Do not process the media_baseurl document property.
  *
- * There are a few differences between AppStream's metainfo files (shipped by upstream projects)
- * and the collection metadata (shipped by distributors).
- * The data source kind indicates which style we should process.
- * Usually you do not want to set this explicitly.
- **/
+ * Influence certain aspects of how AppStream metadata is parsed.
+ */
 typedef enum {
-	AS_FORMAT_STYLE_UNKNOWN,
-	AS_FORMAT_STYLE_METAINFO,
-	AS_FORMAT_STYLE_COLLECTION,
-	/*< private >*/
-	AS_FORMAT_STYLE_LAST
-} AsFormatStyle;
-
-/**
- * AsFormatKind:
- * @AS_FORMAT_KIND_UNKNOWN:		Unknown metadata format.
- * @AS_FORMAT_KIND_XML:			AppStream XML metadata.
- * @AS_FORMAT_KIND_YAML:		AppStream YAML (DEP-11) metadata.
- * @AS_FORMAT_KIND_DESKTOP_ENTRY:	XDG Desktop Entry data.
- *
- * Format of the AppStream metadata.
- **/
-typedef enum {
-	AS_FORMAT_KIND_UNKNOWN,
-	AS_FORMAT_KIND_XML,
-	AS_FORMAT_KIND_YAML,
-	AS_FORMAT_KIND_DESKTOP_ENTRY,
-	/*< private >*/
-	AS_FORMAT_KIND_LAST
-} AsFormatKind;
-
-const gchar		*as_format_kind_to_string (AsFormatKind kind);
-AsFormatKind		 as_format_kind_from_string (const gchar *kind_str);
-
-/**
- * AsFormatVersion:
- * @AS_FORMAT_VERSION_V0_6:	0.6
- * @AS_FORMAT_VERSION_V0_7:	0.7
- * @AS_FORMAT_VERSION_V0_8:	0.8
- * @AS_FORMAT_VERSION_V0_9:	0.9
- * @AS_FORMAT_VERSION_V0_10:	0.10
- * @AS_FORMAT_VERSION_V0_11:	0.11
- * @AS_FORMAT_VERSION_V0_12:	0.12
- *
- * Format version / API level of the AppStream metadata.
- **/
-typedef enum {
-	AS_FORMAT_VERSION_V0_6,
-	AS_FORMAT_VERSION_V0_7,
-	AS_FORMAT_VERSION_V0_8,
-	AS_FORMAT_VERSION_V0_9,
-	AS_FORMAT_VERSION_V0_10,
-	AS_FORMAT_VERSION_V0_11,
-	AS_FORMAT_VERSION_V0_12,
-	/*< private >*/
-	AS_FORMAT_VERSION_LAST
-} AsFormatVersion;
-
-const gchar		*as_format_version_to_string (AsFormatVersion version);
-AsFormatVersion		 as_format_version_from_string (const gchar *version_str);
-
-#define AS_CURRENT_FORMAT_VERSION AS_FORMAT_VERSION_V0_12
+	AS_PARSE_FLAG_NONE = 0,
+	AS_PARSE_FLAG_IGNORE_MEDIABASEURL = 1 << 0
+} AsParseFlags;
 
 /**
  * AsMetadataError:
@@ -207,6 +152,10 @@ const gchar		*as_metadata_get_architecture (AsMetadata *metad);
 void			as_metadata_set_architecture (AsMetadata *metad,
 							const gchar *arch);
 
+AsParseFlags		as_metadata_get_parse_flags (AsMetadata *metad);
+void			as_metadata_set_parse_flags (AsMetadata *metad,
+						     AsParseFlags flags);
+
 G_END_DECLS
 
 #endif /* __AS_METADATA_H */
