@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2019 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2020 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -23,16 +23,18 @@
  * @short_description: Access the AppStream metadata pool.
  *
  * This class loads AppStream metadata from various sources and refines it with existing
- * knowledge about the system (e.g. by setting absolute pazhs for cached icons).
+ * knowledge about the system (e.g. by setting absolute paths for cached icons).
  * An #AsPool will use an on-disk cache to store metadata is has read and refined to
  * speed up the loading time when the same data is requested a second time.
  *
- * You can find AppStream metadata matching farious criteria, and also add new metadata to
- * the pool.
- * The caching behavior can be controlled by the application using #AsPool.
+ * You can find AppStream metadata matching various user-defined criteria, and also add new
+ * metadata to the pool.
+ * The caching behavior can be controlled by the application using #AsCacheFlags.
  *
  * An AppStream cache object can also be created and read using the appstreamcli(1) utility.
  *
+ * This class is threadsafe.
+ *
  * See also: #AsComponent
  */
 
@@ -72,7 +74,7 @@ typedef struct
 	AsCache *system_cache;
 	AsCache *cache;
 	gchar *cache_fname;
-	gchar *system_cache_fname;
+	gchar *sys_cache_dir;
 
 	gchar **term_greylist;
 
@@ -80,8 +82,7 @@ typedef struct
 	AsCacheFlags cache_flags;
 	gboolean prefer_local_metainfo;
 
-	gchar *sys_cache_path;
-	gchar *user_cache_path;
+	GMutex mutex;
 } AsPoolPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsPool, as_pool, G_TYPE_OBJECT)
@@ -125,6 +126,8 @@ as_pool_init (AsPool *pool)
 	guint i;
 	g_autoptr(AsDistroDetails) distro = NULL;
 
+	g_mutex_init (&priv->mutex);
+
 	/* set active locale */
 	priv->locale = as_get_current_locale ();
 
@@ -139,7 +142,7 @@ as_pool_init (AsPool *pool)
 	priv->term_greylist = g_strsplit (AS_SEARCH_GREYLIST_STR, ";", -1);
 
 	/* system-wide cache locations */
-	priv->sys_cache_path = g_strdup (AS_APPSTREAM_CACHE_PATH);
+	priv->sys_cache_dir = g_strdup (AS_APPSTREAM_CACHE_PATH);
 
 	if (as_utils_is_root ()) {
 		/* users umask shouldn't interfere with us creating new files when we are root */
@@ -194,6 +197,7 @@ as_pool_finalize (GObject *object)
 	AsPool *pool = AS_POOL (object);
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
 
+	g_mutex_lock (&priv->mutex);
 	g_free (priv->screenshot_service_url);
 
 	g_ptr_array_unref (priv->xml_dirs);
@@ -203,14 +207,14 @@ as_pool_finalize (GObject *object)
 	g_object_unref (priv->cache);
 	g_object_unref (priv->system_cache);
 	g_free (priv->cache_fname);
+	g_free (priv->sys_cache_dir);
 
 	g_free (priv->locale);
 	g_free (priv->current_arch);
-
 	g_strfreev (priv->term_greylist);
 
-	g_free (priv->sys_cache_path);
-	g_free (priv->user_cache_path);
+	g_mutex_unlock (&priv->mutex);
+	g_mutex_clear (&priv->mutex);
 
 	G_OBJECT_CLASS (as_pool_parent_class)->finalize (object);
 }
@@ -234,6 +238,7 @@ static inline gboolean
 as_pool_can_query_system_cache (AsPool *pool)
 {
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 	if (as_flags_contains (priv->cache_flags, AS_CACHE_FLAG_USE_SYSTEM))
 		return as_cache_is_open (priv->system_cache);
 	return FALSE;
@@ -572,24 +577,40 @@ gboolean
 as_pool_clear2 (AsPool *pool, GError **error)
 {
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 
-	/* close & delete session cache */
-	as_cache_close (priv->cache);
-	if (g_file_test (priv->cache_fname, G_FILE_TEST_EXISTS)) {
-		if (g_remove (priv->cache_fname) != 0) {
-			g_set_error_literal (error,
-						AS_POOL_ERROR,
-						AS_POOL_ERROR_OLD_CACHE,
-						_("Unable to remove old cache."));
-			return FALSE;
-		}
-	}
-
-	/* close system cache so it won't be used anymore */
+	/* close system cache so it won't be used anymore
+	 * (will be loaded explicitly again later, when needed) */
 	as_cache_close (priv->system_cache);
 
-	/* reopen the session cache as a new, pristine one */
-	return as_cache_open (priv->cache, priv->cache_fname, priv->locale, error);
+	/* if we were just created, we may be able to reuse the current temporary cache,
+	 * instead of creating a new one (which is a bit wasteful).
+	 * Reuse requires a temporary cache with no elements. */
+	if ((g_strcmp0 (priv->cache_fname, ":temporary") == 0) &&
+	    (as_cache_count_components (priv->cache, NULL) == 0)) {
+		/* we can reuse the user cache */
+
+		g_debug ("Not clearing user cache: The cache was already empty.");
+		return TRUE;
+	} else {
+		/* it looks like we can not reuse the old cache, so now we need to clear
+		 * the cache for real by deleting the old one and creating a new one */
+
+		g_debug ("Clearing user cache.");
+		as_cache_close (priv->cache);
+		if (g_file_test (priv->cache_fname, G_FILE_TEST_EXISTS)) {
+			if (g_remove (priv->cache_fname) != 0) {
+				g_set_error_literal (error,
+							AS_POOL_ERROR,
+							AS_POOL_ERROR_OLD_CACHE,
+							_("Unable to remove old cache."));
+				return FALSE;
+			}
+		}
+
+		/* reopen the session cache as a new, pristine one */
+		return as_cache_open (priv->cache, priv->cache_fname, priv->locale, error);
+	}
 }
 
 /**
@@ -652,6 +673,7 @@ as_pool_has_system_metadata_paths (AsPool *pool)
 {
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
 	guint i;
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 
 	for (i = 0; i < priv->xml_dirs->len; i++) {
 		const gchar *dir = (const gchar*) g_ptr_array_index (priv->xml_dirs, i);
@@ -675,6 +697,7 @@ as_pool_metadata_changed (AsPool *pool, AsCache *cache, gboolean system_only)
 {
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
 	guint i;
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 
 	for (i = 0; i < priv->xml_dirs->len; i++) {
 		const gchar *dir = (const gchar*) g_ptr_array_index (priv->xml_dirs, i);
@@ -702,6 +725,7 @@ as_pool_metadata_changed (AsPool *pool, AsCache *cache, gboolean system_only)
 static gboolean
 as_pool_load_collection_data (AsPool *pool, gboolean refresh, GError **error)
 {
+	AsPoolPrivate *priv = GET_PRIVATE (pool);
 	GPtrArray *cpts;
 	g_autoptr(GPtrArray) merge_cpts = NULL;
 	guint i;
@@ -709,23 +733,29 @@ as_pool_load_collection_data (AsPool *pool, gboolean refresh, GError **error)
 	g_autoptr(AsMetadata) metad = NULL;
 	g_autoptr(GPtrArray) mdata_files = NULL;
 	GError *tmp_error = NULL;
-	AsPoolPrivate *priv = GET_PRIVATE (pool);
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	/* see if we can use the system caches */
 	if (!refresh && as_pool_has_system_metadata_paths (pool)) {
 		g_autofree gchar *fname = NULL;
-		fname = g_strdup_printf ("%s/%s.cache", priv->sys_cache_path, priv->locale);
+		g_mutex_lock (&priv->mutex);
+		fname = g_strdup_printf ("%s/%s.cache", priv->sys_cache_dir, priv->locale);
 		as_cache_set_location (priv->system_cache, fname);
+		g_mutex_unlock (&priv->mutex);
 
 		if (!as_pool_metadata_changed (pool, priv->system_cache, TRUE)) {
 			g_debug ("System metadata cache seems up to date.");
 
+			g_mutex_lock (&priv->mutex);
 			if (as_flags_contains (priv->cache_flags, AS_CACHE_FLAG_USE_SYSTEM)) {
+				g_mutex_unlock (&priv->mutex);
 				g_debug ("Using system cache data.");
 				if (g_file_test (fname, G_FILE_TEST_EXISTS)) {
 					as_cache_close (priv->system_cache);
 
+					g_mutex_lock (&priv->mutex);
 					if (as_cache_open2 (priv->system_cache, priv->locale, &tmp_error)) {
+						g_mutex_unlock (&priv->mutex);
 						return TRUE;
 					} else {
 						/* if we can't open the system cache for whatever reason, we complain but
@@ -734,10 +764,12 @@ as_pool_load_collection_data (AsPool *pool, gboolean refresh, GError **error)
 						g_error_free (tmp_error);
 						tmp_error = NULL;
 					}
+					g_mutex_unlock (&priv->mutex);
 				} else {
 					g_debug ("Missing cache for language '%s', attempting to load fresh data.", priv->locale);
 				}
 			} else {
+				g_mutex_unlock (&priv->mutex);
 				g_debug ("Not using system cache.");
 				as_cache_close (priv->system_cache);
 			}
@@ -752,12 +784,17 @@ as_pool_load_collection_data (AsPool *pool, gboolean refresh, GError **error)
 	/* prepare metadata parser */
 	metad = as_metadata_new ();
 	as_metadata_set_format_style (metad, AS_FORMAT_STYLE_COLLECTION);
+	g_mutex_lock (&priv->mutex);
 	as_metadata_set_locale (metad, priv->locale);
+	g_mutex_unlock (&priv->mutex);
 
 	/* find AppStream metadata */
 	ret = TRUE;
 	mdata_files = g_ptr_array_new_with_free_func (g_free);
 
+	/* protect access to directory lists */
+	locker = g_mutex_locker_new (&priv->mutex);
+
 	/* find XML data */
 	for (i = 0; i < priv->xml_dirs->len; i++) {
 		const gchar *xml_path = (const gchar *) g_ptr_array_index (priv->xml_dirs, i);
@@ -800,6 +837,9 @@ as_pool_load_collection_data (AsPool *pool, gboolean refresh, GError **error)
 		}
 	}
 
+	/* no need for further locking after this point, AsCache is threadsafe */
+	g_clear_pointer (&locker, g_mutex_locker_free);
+
 	/* parse the found data */
 	for (i = 0; i < mdata_files->len; i++) {
 		g_autoptr(GFile) infile = NULL;
@@ -855,7 +895,7 @@ as_pool_load_collection_data (AsPool *pool, gboolean refresh, GError **error)
 			continue;
 		}
 
-		as_pool_add_component (pool, cpt, &tmp_error);
+		as_pool_add_component_internal (pool, cpt, TRUE, &tmp_error);
 		if (tmp_error != NULL) {
 			g_debug ("Metadata ignored: %s", tmp_error->message);
 			g_error_free (tmp_error);
@@ -868,7 +908,7 @@ as_pool_load_collection_data (AsPool *pool, gboolean refresh, GError **error)
 	for (i = 0; i < merge_cpts->len; i++) {
 		AsComponent *mcpt = AS_COMPONENT (g_ptr_array_index (merge_cpts, i));
 
-		as_pool_add_component (pool, mcpt, &tmp_error);
+		as_pool_add_component_internal (pool, mcpt, TRUE, &tmp_error);
 		if (tmp_error != NULL) {
 			g_debug ("Merge component ignored: %s", tmp_error->message);
 			g_error_free (tmp_error);
@@ -898,7 +938,9 @@ as_pool_get_desktop_entries_table (AsPool *pool)
 
 	/* prepare metadata parser */
 	metad = as_metadata_new ();
+	g_mutex_lock (&priv->mutex);
 	as_metadata_set_locale (metad, priv->locale);
+	g_mutex_unlock (&priv->mutex);
 
 	de_cpt_table = g_hash_table_new_full (g_str_hash,
 					      g_str_equal,
@@ -968,7 +1010,9 @@ as_pool_load_metainfo_data (AsPool *pool, GHashTable *desktop_entry_cpts)
 
 	/* prepare metadata parser */
 	metad = as_metadata_new ();
+	g_mutex_lock (&priv->mutex);
 	as_metadata_set_locale (metad, priv->locale);
+	g_mutex_unlock (&priv->mutex);
 
 	/* find metainfo files */
 	g_debug ("Searching for data in: %s", METAINFO_DIR);
@@ -1082,23 +1126,34 @@ as_pool_load_metainfo_desktop_data (AsPool *pool)
 	g_autoptr(GHashTable) de_cpts = NULL;
 
 	/* check if we actually need to load anything */
-	if (!as_flags_contains (priv->flags, AS_POOL_FLAG_READ_DESKTOP_FILES) && !as_flags_contains (priv->flags, AS_POOL_FLAG_READ_METAINFO))
+	g_mutex_lock (&priv->mutex);
+	if (!as_flags_contains (priv->flags, AS_POOL_FLAG_READ_DESKTOP_FILES) && !as_flags_contains (priv->flags, AS_POOL_FLAG_READ_METAINFO)) {
+		g_mutex_unlock (&priv->mutex);
 		return;
+	}
+	g_mutex_unlock (&priv->mutex);
 
 	/* get a hashmap of desktop-entry components */
 	de_cpts = as_pool_get_desktop_entries_table (pool);
 
+	g_mutex_lock (&priv->mutex);
 	if (as_flags_contains (priv->flags, AS_POOL_FLAG_READ_METAINFO)) {
+		g_mutex_unlock (&priv->mutex);
+
 		/* load metainfo components, absorb desktop-entry components into them */
 		as_pool_load_metainfo_data (pool, de_cpts);
+	} else {
+		g_mutex_unlock (&priv->mutex);
 	}
 
 	/* read all remaining .desktop file components, if needed */
+	g_mutex_lock (&priv->mutex);
 	if (as_flags_contains (priv->flags, AS_POOL_FLAG_READ_DESKTOP_FILES)) {
 		GHashTableIter iter;
 		gpointer value;
 		GError *error = NULL;
 
+		g_mutex_unlock (&priv->mutex);
 		g_debug ("Including components from .desktop files in the pool.");
 		g_hash_table_iter_init (&iter, de_cpts);
 		while (g_hash_table_iter_next (&iter, NULL, &value)) {
@@ -1111,6 +1166,8 @@ as_pool_load_metainfo_desktop_data (AsPool *pool)
 				error = NULL;
 			}
 		}
+	} else {
+		g_mutex_unlock (&priv->mutex);
 	}
 }
 
@@ -1126,6 +1183,7 @@ as_pool_cache_refine_component_cb (gpointer data, gpointer user_data)
 	AsPool *pool = AS_POOL (user_data);
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
 	AsComponent *cpt = AS_COMPONENT (data);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 
 	/* add additional data to the component, e.g. external screenshots. Also refines
 	 * the component's icon paths */
@@ -1137,6 +1195,7 @@ as_pool_cache_refine_component_cb (gpointer data, gpointer user_data)
 /**
  * as_pool_load:
  * @pool: An instance of #AsPool.
+ * @cancellable: a #GCancellable.
  * @error: A #GError or %NULL.
  *
  * Builds an index of all found components in the watched locations.
@@ -1158,12 +1217,19 @@ as_pool_load (AsPool *pool, GCancellable *cancellable, GError **error)
 	gdouble valid_percentage;
 	GError *tmp_error = NULL;
 
+	g_mutex_lock (&priv->mutex);
 	if (as_flags_contains (priv->cache_flags, AS_CACHE_FLAG_NO_CLEAR)) {
+		g_autoptr(GMutexLocker) inner_locker = NULL;
+		g_mutex_unlock (&priv->mutex);
+		inner_locker = g_mutex_locker_new (&priv->mutex);
+
 		/* we are supposed not to clear the cache before laoding its data */
 		if (!as_cache_open (priv->cache, priv->cache_fname, priv->locale, error))
 			return FALSE;
 	} else {
-		/* load (usually) means to reload, so we clear potential old data */
+		g_mutex_unlock (&priv->mutex);
+
+		/* load (here) means to reload, so we clear potential old data */
 		if (!as_pool_clear2 (pool, error))
 			return FALSE;
 	}
@@ -1171,8 +1237,13 @@ as_pool_load (AsPool *pool, GCancellable *cancellable, GError **error)
 	as_cache_make_floating (priv->cache);
 
 	/* read all AppStream metadata that we can find */
-	if (as_flags_contains (priv->flags, AS_POOL_FLAG_READ_COLLECTION))
+	g_mutex_lock (&priv->mutex);
+	if (as_flags_contains (priv->flags, AS_POOL_FLAG_READ_COLLECTION)) {
+		g_mutex_unlock (&priv->mutex);
 		ret = as_pool_load_collection_data (pool, FALSE, error);
+	} else {
+		g_mutex_unlock (&priv->mutex);
+	}
 
 	/* read all metainfo and desktop files and add them to the pool */
 	as_pool_load_metainfo_desktop_data (pool);
@@ -1215,6 +1286,67 @@ as_pool_load (AsPool *pool, GCancellable *cancellable, GError **error)
 	return ret;
 }
 
+static void
+_pool_load_thread (GTask *task,
+		   gpointer source_object,
+		   gpointer task_data,
+		   GCancellable *cancellable)
+{
+	AsPool *pool = AS_POOL (source_object);
+	GError *error = NULL;
+	gboolean success;
+
+	success = as_pool_load (pool, cancellable, &error);
+	if (error != NULL)
+		g_task_return_error (task, error);
+	else
+		g_task_return_boolean (task, success);
+}
+
+/**
+ * as_pool_load_async:
+ * @pool: An instance of #AsPool.
+ * @cancellable: a #GCancellable.
+ * @callback: A #GAsyncReadyCallback
+ * @user_data: Data to pass to @callback
+ *
+ * Asynchronously loads data from all registered locations.
+ * Equivalent to as_pool_load() (but asynchronous)
+ *
+ * Since: 0.12.10
+ **/
+void
+as_pool_load_async (AsPool *pool,
+		    GCancellable *cancellable,
+		    GAsyncReadyCallback callback,
+		    gpointer user_data)
+{
+	GTask *task = g_task_new (pool, cancellable, callback, user_data);
+	g_task_run_in_thread (task, _pool_load_thread);
+	g_object_unref (task);
+}
+
+/**
+ * as_pool_load_finish:
+ * @pool: An instance of #AsPool.
+ * @result: A #GAsyncResult
+ * @error: A #GError or %NULL.
+ *
+ * Retrieve the result of as_pool_load_async().
+ *
+ * Returns: %TRUE for success
+ *
+ * Since: 0.12.10
+ **/
+gboolean
+as_pool_load_finish (AsPool *pool,
+		     GAsyncResult *result,
+		     GError **error)
+{
+	g_return_val_if_fail (g_task_is_valid (result, pool), FALSE);
+	return g_task_propagate_boolean (G_TASK (result), error);
+}
+
 /**
  * as_pool_load_cache_file:
  * @pool: An instance of #AsPool.
@@ -1227,6 +1359,7 @@ gboolean
 as_pool_load_cache_file (AsPool *pool, const gchar *fname, GError **error)
 {
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 
 	as_cache_close (priv->system_cache);
 	if (!as_cache_open (priv->cache, fname, priv->locale, error))
@@ -1249,11 +1382,14 @@ as_pool_save_cache_file (AsPool *pool, const gchar *fname, GError **error)
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
 	g_autoptr(GPtrArray) cpts = NULL;
 	g_autoptr(AsCache) cache = NULL;
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	cpts = as_pool_get_components (pool);
 	cache = as_cache_new ();
+	locker = g_mutex_locker_new (&priv->mutex);
 	if (!as_cache_open (cache, fname, priv->locale, error))
 		return FALSE;
+	g_clear_pointer (&locker, g_mutex_locker_free);
 
 	for (guint i = 0; i < cpts->len; i++) {
 		if (!as_cache_insert (cache, AS_COMPONENT (g_ptr_array_index (cpts, i)), error))
@@ -1269,7 +1405,7 @@ as_pool_save_cache_file (AsPool *pool, const gchar *fname, GError **error)
  *
  * Get a list of found components.
  *
- * Returns: (transfer container) (element-type AsComponent): an array of #AsComponent instances.
+ * Returns: (transfer full) (element-type AsComponent): an array of #AsComponent instances.
  */
 GPtrArray*
 as_pool_get_components (AsPool *pool)
@@ -1306,7 +1442,7 @@ as_pool_get_components (AsPool *pool)
  * This function may contain multiple results if we have
  * data describing this component from multiple scopes/origin types.
  *
- * Returns: (transfer container) (element-type AsComponent): An #AsComponent
+ * Returns: (transfer full) (element-type AsComponent): An #AsComponent
  */
 GPtrArray*
 as_pool_get_components_by_id (AsPool *pool, const gchar *cid)
@@ -1342,7 +1478,7 @@ as_pool_get_components_by_id (AsPool *pool, const gchar *cid)
  *
  * Find components in the AppStream data pool which provide a certain item.
  *
- * Returns: (transfer container) (element-type AsComponent): an array of #AsComponent objects which have been found.
+ * Returns: (transfer full) (element-type AsComponent): an array of #AsComponent objects which have been found.
  */
 GPtrArray*
 as_pool_get_components_by_provided_item (AsPool *pool,
@@ -1379,7 +1515,7 @@ as_pool_get_components_by_provided_item (AsPool *pool,
  *
  * Return a list of all components in the pool which are of a certain kind.
  *
- * Returns: (transfer container) (element-type AsComponent): an array of #AsComponent objects which have been found.
+ * Returns: (transfer full) (element-type AsComponent): an array of #AsComponent objects which have been found.
  */
 GPtrArray*
 as_pool_get_components_by_kind (AsPool *pool, AsComponentKind kind)
@@ -1414,7 +1550,7 @@ as_pool_get_components_by_kind (AsPool *pool, AsComponentKind kind)
  *
  * Return a list of components which are in one of the categories.
  *
- * Returns: (transfer container) (element-type AsComponent): an array of #AsComponent objects which have been found.
+ * Returns: (transfer full) (element-type AsComponent): an array of #AsComponent objects which have been found.
  */
 GPtrArray*
 as_pool_get_components_by_categories (AsPool *pool, gchar **categories)
@@ -1458,7 +1594,7 @@ as_pool_get_components_by_categories (AsPool *pool, gchar **categories)
  * Find components in the AppStream data pool which provide a specific launchable.
  * See #AsLaunchable for details on launchables, or refer to the AppStream specification.
  *
- * Returns: (transfer container) (element-type AsComponent): an array of #AsComponent objects which have been found.
+ * Returns: (transfer full) (element-type AsComponent): an array of #AsComponent objects which have been found.
  *
  * Since: 0.11.4
  */
@@ -1563,10 +1699,13 @@ as_pool_build_search_terms (AsPool *pool, const gchar *search)
 	idx = 0;
 	stemmer = g_object_ref (as_stemmer_get ());
 	for (i = 0; strv[i] != NULL; i++) {
+		gchar *token;
 		if (!as_user_search_term_valid (strv[i]))
 			continue;
 		/* stem the string and add it to terms */
-		terms[idx++] = as_stemmer_stem (stemmer, strv[i]);
+		token = as_stemmer_stem (stemmer, strv[i]);
+		if (token != NULL)
+			terms[idx++] = token;
 	}
 	/* if we have no valid terms, return NULL */
 	if (idx == 0) {
@@ -1620,7 +1759,9 @@ as_pool_search (AsPool *pool, const gchar *search)
 
 	result = as_cache_search (priv->cache, terms, FALSE, &tmp_error);
 	if (result == NULL) {
+		g_mutex_lock (&priv->mutex);
 		g_warning ("Search in session cache failed: %s", tmp_error->message);
+		g_mutex_unlock (&priv->mutex);
 		return g_ptr_array_new_with_free_func (g_object_unref);
 	}
 
@@ -1674,24 +1815,26 @@ as_pool_refresh_system_cache (AsPool *pool, gboolean force, GError **error)
 	gboolean ret = FALSE;
 	g_autofree gchar *cache_fname = NULL;
 	g_autoptr(GError) data_load_error = NULL;
-	g_autoptr(GError) tmp_error = NULL;
+	GError *tmp_error = NULL;
 	AsCacheFlags prev_cache_flags;
 	guint invalid_cpts_n;
 
 	/* try to create cache directory, in case it doesn't exist */
-	g_mkdir_with_parents (priv->sys_cache_path, 0755);
-	if (!as_utils_is_writable (priv->sys_cache_path)) {
+	g_mkdir_with_parents (priv->sys_cache_dir, 0755);
+	if (!as_utils_is_writable (priv->sys_cache_dir)) {
 		g_set_error (error,
 				AS_POOL_ERROR,
 				AS_POOL_ERROR_TARGET_NOT_WRITABLE,
-				_("Cache location '%s' is not writable."), priv->sys_cache_path);
+				_("Cache location '%s' is not writable."), priv->sys_cache_dir);
 		return FALSE;
 	}
 
 	/* create the filename of our cache and set the location of the system cache.
 	 * This has to happen before we check for new metadata, so the system cache can
 	 * determine its age (so we know whether a refresh is needed at all). */
-	cache_fname = g_strdup_printf ("%s/%s.cache", priv->sys_cache_path, priv->locale);
+	g_mutex_lock (&priv->mutex);
+	cache_fname = g_strdup_printf ("%s/%s.cache", priv->sys_cache_dir, priv->locale);
+	g_mutex_unlock (&priv->mutex);
 	as_cache_set_location (priv->system_cache, cache_fname);
 
 	/* collect metadata */
@@ -1701,8 +1844,7 @@ as_pool_refresh_system_cache (AsPool *pool, gboolean force, GError **error)
 	if (tmp_error != NULL) {
 		/* the exact error is not forwarded here, since we might be able to partially update the cache */
 		g_warning ("Error while collecting metadata: %s", tmp_error->message);
-		g_error_free (tmp_error);
-		tmp_error = NULL;
+		g_clear_error (&tmp_error);
 	}
 #endif
 
@@ -1727,15 +1869,21 @@ as_pool_refresh_system_cache (AsPool *pool, gboolean force, GError **error)
 
 	/* open system cache as user cache temporarily, so we can modify it */
 	g_remove (cache_fname);
-	if (!as_cache_open (priv->cache, cache_fname, priv->locale, error))
+	g_mutex_lock (&priv->mutex);
+	if (!as_cache_open (priv->cache, cache_fname, priv->locale, error)) {
+		g_mutex_unlock (&priv->mutex);
 		return FALSE;
+	}
+	g_mutex_unlock (&priv->mutex);
 
 	/* NOTE: we will only cache AppStream metadata, no .desktop file metadata etc. */
 
 	/* since the session cache is the system cache now (in order to update it), temporarily modify
 	 * the cache flags */
+	g_mutex_lock (&priv->mutex);
 	prev_cache_flags = priv->cache_flags;
 	priv->cache_flags = AS_CACHE_FLAG_USE_USER;
+	g_mutex_unlock (&priv->mutex);
 
 	/* set cache to floating mode to increase performance by holding all data
 	 *in memory in a serialized form */
@@ -1757,7 +1905,9 @@ as_pool_refresh_system_cache (AsPool *pool, gboolean force, GError **error)
 	as_cache_close (priv->cache);
 
 	/* restore cache flags */
+	g_mutex_lock (&priv->mutex);
 	priv->cache_flags = prev_cache_flags;
+	g_mutex_unlock (&priv->mutex);
 
 	/* switch back to default sync mode */
 	as_cache_set_nosync (priv->cache, FALSE);
@@ -1801,6 +1951,8 @@ void
 as_pool_set_locale (AsPool *pool, const gchar *locale)
 {
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
+
 	g_free (priv->locale);
 	priv->locale = g_strdup (locale);
 }
@@ -1817,6 +1969,7 @@ const gchar *
 as_pool_get_locale (AsPool *pool)
 {
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 	return priv->locale;
 }
 
@@ -1834,12 +1987,16 @@ as_pool_add_metadata_location_internal (AsPool *pool, const gchar *directory, gb
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
 	gboolean dir_added = FALSE;
 	gchar *path;
+	g_autoptr(GMutexLocker) locker = NULL;
 
 	if (!g_file_test (directory, G_FILE_TEST_IS_DIR)) {
 		g_debug ("Not adding metadata location '%s': Is no directory", directory);
 		return;
 	}
 
+	/* protect access to directory arrays */
+	locker = g_mutex_locker_new (&priv->mutex);
+
 	/* metadata locations */
 	path = g_build_filename (directory, "xml", NULL);
 	if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
@@ -1909,6 +2066,7 @@ void
 as_pool_clear_metadata_locations (AsPool *pool)
 {
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 
 	/* clear arrays */
 	g_ptr_array_set_size (priv->xml_dirs, 0);
@@ -1928,6 +2086,7 @@ AsCacheFlags
 as_pool_get_cache_flags (AsPool *pool)
 {
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 	return priv->cache_flags;
 }
 
@@ -1942,6 +2101,7 @@ void
 as_pool_set_cache_flags (AsPool *pool, AsCacheFlags flags)
 {
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 	priv->cache_flags = flags;
 }
 
@@ -1955,6 +2115,7 @@ AsPoolFlags
 as_pool_get_flags (AsPool *pool)
 {
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 	return priv->flags;
 }
 
@@ -1969,6 +2130,7 @@ void
 as_pool_set_flags (AsPool *pool, AsPoolFlags flags)
 {
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 	priv->flags = flags;
 }
 
@@ -2000,6 +2162,8 @@ void
 as_pool_set_cache_location (AsPool *pool, const gchar *fname)
 {
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
+
 	g_free (priv->cache_fname);
 	priv->cache_fname = g_strdup (fname);
 }
@@ -2016,6 +2180,7 @@ const gchar*
 as_pool_get_cache_location (AsPool *pool)
 {
 	AsPoolPrivate *priv = GET_PRIVATE (pool);
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
 	return priv->cache_fname;
 }
 
