@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2016 Richard Hughes <richard@hughsie.com>
- * Copyright (C) 2016 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2016-2020 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -75,8 +75,8 @@ as_spdx_license_tokenize_drop (AsSpdxHelper *helper)
 	if (helper->collect->len == 0)
 		return;
 
-	/* is license enum */
-	if (as_is_spdx_license_id (tmp)) {
+	/* is license or exception enum */
+	if (as_is_spdx_license_id (tmp) || as_is_spdx_license_exception_id (tmp)) {
 		g_ptr_array_add (helper->array, g_strdup_printf ("@%s", tmp));
 		helper->last_token_literal = FALSE;
 		g_string_truncate (helper->collect, 0);
@@ -122,6 +122,14 @@ as_spdx_license_tokenize_drop (AsSpdxHelper *helper)
 		return;
 	}
 
+	/* is extension to the license */
+	if (g_strcmp0 (tmp, "with") == 0 || g_strcmp0 (tmp, "WITH") == 0) {
+		g_ptr_array_add (helper->array, g_strdup ("^"));
+		helper->last_token_literal = FALSE;
+		g_string_truncate (helper->collect, 0);
+		return;
+	}
+
 	/* is literal */
 	if (helper->last_token_literal) {
 		last_literal = g_strdup (_g_ptr_array_last (helper->array));
@@ -141,7 +149,7 @@ as_spdx_license_tokenize_drop (AsSpdxHelper *helper)
  *
  * Searches the known list of SPDX license IDs.
  *
- * Returns: %TRUE if the icon is a valid "SPDX license ID"
+ * Returns: %TRUE if the string is a valid SPDX license ID
  *
  * Since: 0.9.8
  **/
@@ -171,12 +179,44 @@ as_is_spdx_license_id (const gchar *license_id)
 	return g_strstr_len (g_bytes_get_data (data, NULL), -1, key) != NULL;
 }
 
+/**
+ * as_is_spdx_license_exception_id:
+ * @exception_id: a single SPDX license exception ID, e.g. "GCC-exception-3.1"
+ *
+ * Searches the known list of SPDX license exception IDs.
+ *
+ * Returns: %TRUE if the string is a valid SPDX license exception ID
+ *
+ * Since: 0.12.10
+ **/
+gboolean
+as_is_spdx_license_exception_id (const gchar *exception_id)
+{
+	g_autoptr(GBytes) data = NULL;
+	g_autofree gchar *key = NULL;
+
+	/* handle invalid */
+	if (exception_id == NULL || exception_id[0] == '\0')
+		return FALSE;
+
+	/* load the readonly data section and look for the icon name */
+	data = g_resource_lookup_data (as_get_resource (),
+				       "/org/freedesktop/appstream/spdx-license-exception-ids.txt",
+				       G_RESOURCE_LOOKUP_FLAGS_NONE,
+				       NULL);
+	if (data == NULL)
+		return FALSE;
+	key = g_strdup_printf ("\n%s\n", exception_id);
+
+	return g_strstr_len (g_bytes_get_data (data, NULL), -1, key) != NULL;
+}
+
 /**
  * as_is_spdx_license_expression:
  * @license: a SPDX license string, e.g. "CC-BY-3.0 and GFDL-1.3"
  *
  * Checks the licence string to check it being a valid licence.
- * NOTE: SPDX licences can't typically contain brackets.
+ * NOTE: SPDX licenses can't typically contain brackets.
  *
  * Returns: %TRUE if the icon is a valid "SPDX license"
  *
@@ -187,6 +227,7 @@ as_is_spdx_license_expression (const gchar *license)
 {
 	guint i;
 	g_auto(GStrv) tokens = NULL;
+	gboolean expect_exception = FALSE;
 
 	/* handle nothing set */
 	if (license == NULL || license[0] == '\0')
@@ -203,10 +244,17 @@ as_is_spdx_license_expression (const gchar *license)
 	tokens = as_spdx_license_tokenize (license);
 	if (tokens == NULL)
 		return FALSE;
+
 	for (i = 0; tokens[i] != NULL; i++) {
 		if (tokens[i][0] == '@') {
-			if (as_is_spdx_license_id (tokens[i] + 1))
-				continue;
+			if (expect_exception) {
+				expect_exception = FALSE;
+				if (as_is_spdx_license_exception_id (tokens[i] + 1))
+					continue;
+			} else {
+				if (as_is_spdx_license_id (tokens[i] + 1))
+					continue;
+			}
 		}
 		if (as_is_spdx_license_id (tokens[i]))
 			continue;
@@ -216,6 +264,10 @@ as_is_spdx_license_expression (const gchar *license)
 			continue;
 		if (g_strcmp0 (tokens[i], "+") == 0)
 			continue;
+		if (g_strcmp0 (tokens[i], "^") == 0) {
+			expect_exception = TRUE;
+			continue;
+		}
 		return FALSE;
 	}
 
@@ -260,9 +312,10 @@ as_utils_spdx_license_2to3 (const gchar *license2)
  * @license: a license string, e.g. "LGPLv2+ and (QPL or GPLv2) and MIT"
  *
  * Tokenizes the SPDX license string (or any simarly formatted string)
- * into parts. Any licence parts of the string e.g. "LGPL-2.0+" are prefexed
- * with "@", the conjunctive replaced with "&" and the disjunctive replaced
- * with "|". Brackets are added as indervidual tokens and other strings are
+ * into parts. Any license parts of the string e.g. "LGPL-2.0+" are prefexed
+ * with "@", the conjunctive replaced with "&", the disjunctive replaced
+ * with "|" and the WITH operator for license exceptions replaced with "^".
+ * Brackets are added as indervidual tokens and other strings are
  * appended into single tokens where possible.
  *
  * Returns: (transfer full) (nullable): array of strings, or %NULL for invalid
@@ -344,6 +397,10 @@ as_spdx_license_detokenize (gchar **license_tokens)
 			g_string_append (tmp, " OR ");
 			continue;
 		}
+		if (g_strcmp0 (license_tokens[i], "^") == 0) {
+			g_string_append (tmp, " WITH ");
+			continue;
+		}
 		if (g_strcmp0 (license_tokens[i], "+") == 0) {
 			g_string_append (tmp, "+");
 			continue;
@@ -497,12 +554,18 @@ as_validate_is_content_license_id (const gchar *license_id)
 		return TRUE;
 	if (g_strcmp0 (license_id, "@FSFUL") == 0)
 		return TRUE;
+
+	/* any operators are fine */
 	if (g_strcmp0 (license_id, "&") == 0)
 		return TRUE;
 	if (g_strcmp0 (license_id, "|") == 0)
 		return TRUE;
 	if (g_strcmp0 (license_id, "+") == 0)
 		return TRUE;
+
+	/* if there is any license exception involved, we don't have a content license */
+	if (g_strcmp0 (license_id, "^") == 0)
+		return FALSE;
 	return FALSE;
 }
 
@@ -556,6 +619,9 @@ as_get_license_url (const gchar *license)
 	g_autoptr(GString) license_id = NULL;
 	g_autofree gchar *tmp_spdx = NULL;
 
+	if (license == NULL)
+		return NULL;
+
 	license_id = as_utils_spdx_license_2to3 (license);
 	if (g_str_has_prefix (license_id->str, "@"))
 		g_string_erase (license_id, 0, 1);
@@ -563,8 +629,6 @@ as_get_license_url (const gchar *license)
 	g_string_truncate (license_id, 0);
 	g_string_append (license_id, tmp_spdx);
 
-	if (!as_is_spdx_license_id (license_id->str))
-		return NULL;
 	if (g_str_has_prefix (license_id->str, "LicenseRef")) {
 		gchar *l;
 		/* a license ref may include an URL on its own */
@@ -576,6 +640,65 @@ as_get_license_url (const gchar *license)
 			return NULL;
 		return g_strdup (l);
 	}
+	if (!as_is_spdx_license_id (license_id->str) && !as_is_spdx_license_exception_id (license_id->str))
+		return NULL;
 
 	return g_strdup_printf ("https://spdx.org/licenses/%s.html#page", license_id->str);
 }
+
+/**
+ * as_license_is_free_license:
+ * @license: The SPDX license string to test.
+ *
+ * Check if the given license is for free-as-in-freedom software.
+ * Currently, all licenses listed on the SPDX website are considered to
+ * be "free software" licenses.
+ *
+ * This definition may be tightened in future. In any case, this function does
+ * not give any legal advice. Please read the license texts to know more about
+ * the individual licenses.
+ *
+ * Returns: %TRUE if the license string contains only free-as-in-freedom licenses.
+ *
+ * Since: 0.12.10
+ */
+gboolean
+as_license_is_free_license (const gchar *license)
+{
+	g_auto(GStrv) tokens = NULL;
+	gboolean is_free;
+
+	/* assume we have a free software license, unless proven otherwise */
+	is_free = TRUE;
+	tokens = as_spdx_license_tokenize (license);
+	for (guint i = 0; tokens[i] != NULL; i++) {
+		if (g_strcmp0 (tokens[i], "&") == 0 ||
+		    g_strcmp0 (tokens[i], "+") == 0 ||
+		    g_strcmp0 (tokens[i], "|") == 0 ||
+		    g_strcmp0 (tokens[i], "^") == 0 ||
+		    g_strcmp0 (tokens[i], "(") == 0 ||
+		    g_strcmp0 (tokens[i], ")") == 0)
+			continue;
+
+		if (g_str_has_prefix (tokens[i], "@LicenseRef")) {
+			/* we only consider license ref's to be free if they explicitly state so */
+			if (!g_str_has_prefix (tokens[i], "@LicenseRef-free")) {
+				is_free = FALSE;
+				break;
+			}
+		} else if (g_str_has_prefix (tokens[i], "@NOASSERTION")
+			|| g_str_has_prefix (tokens[i], "@NONE")) {
+			/* no license info is fishy as well */
+			is_free = FALSE;
+			break;
+		}
+
+		if (tokens[i][0] != '@') {
+			/* if token has no license-id prefix, consider the license to be non-free */
+			is_free = FALSE;
+			break;
+		}
+	}
+
+	return is_free;
+}
