@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2016 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2020 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -18,6 +18,10 @@
  * along with this library.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#if !defined (__APPSTREAM_H) && !defined (AS_COMPILATION)
+#error "Only <appstream.h> can be included directly."
+#endif
+
 #ifndef __AS_SPDX_H
 #define __AS_SPDX_H
 
@@ -25,8 +29,9 @@
 
 G_BEGIN_DECLS
 
-gboolean	 as_is_spdx_license_id (const gchar *license_id);
-gboolean	 as_is_spdx_license_expression (const gchar *license);
+gboolean	as_is_spdx_license_id (const gchar *license_id);
+gboolean	as_is_spdx_license_exception_id (const gchar *exception_id);
+gboolean	as_is_spdx_license_expression (const gchar *license);
 
 gchar		**as_spdx_license_tokenize (const gchar *license);
 gchar		*as_spdx_license_detokenize (gchar **license_tokens);
@@ -34,6 +39,9 @@ gchar		*as_spdx_license_detokenize (gchar **license_tokens);
 gchar		*as_license_to_spdx_id (const gchar *license);
 
 gboolean	as_license_is_metadata_license (const gchar *license);
+gboolean	as_license_is_free_license (const gchar *license);
+
+gchar		*as_get_license_url (const gchar *license);
 
 G_END_DECLS
 
