@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2014-2018 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2014-2020 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -66,6 +66,7 @@ G_BEGIN_DECLS
  * @AS_TAG_RECOMMENDS:			`recommends` / `Recommends`
  * @AS_TAG_REQUIRES:			`requires` / `Requires`
  * @AS_TAG_AGREEMENT:			`agreement` / `Àgreement`
+ * @AS_TAG_NAME_VARIANT_SUFFIX:		`name_variant_suffix` / `NameVariantSuffix`
  * @AS_TAG_P:				Description markup `p`
  * @AS_TAG_LI:				Description markup `li`
  * @AS_TAG_OL:				Description markup `ol`
@@ -108,6 +109,7 @@ typedef enum {
 	AS_TAG_RECOMMENDS,
 	AS_TAG_REQUIRES,
 	AS_TAG_AGREEMENT,
+	AS_TAG_NAME_VARIANT_SUFFIX,
 	AS_TAG_P,
 	AS_TAG_LI,
 	AS_TAG_UL,
